@@ -18,6 +18,7 @@
 import glob
 import logging
 import os
+import random
 import shlex
 import shutil
 import signal
@@ -103,7 +104,8 @@ def parse_command_line(args):
                           dest='x11',
                           action='store_const',
                           const=X11Mode.AUTO,
-                          help='enable nested X11 using X2Go nxagent or Xephry or Xnest or Xvfb'
+                          help='enable nested X11 using X2Go nxagent or Xephry or Xnest'
+                          ' but not Xvfb'
                           ' (default: X11 disabled)')
     x11_args.add_argument('--nxagent',
                           dest='x11',
@@ -200,10 +202,11 @@ class X11Context:
     @staticmethod
     def detect_nested():
         tests = [
+            # NOTE: No Xvfb here because this is meant to be about
+            #       options with user-visible Windows
             ['nxagent', X11Mode.NXAGENT],
             ['Xephyr', X11Mode.XEPHYR],
             ['Xnest', X11Mode.XNEST],
-            ['Xvfb', X11Mode.XVFB],
         ]
         for command, mode in tests:
             if shutil.which(command) is not None:
@@ -301,6 +304,10 @@ class MountTask:
     required: bool = True
 
 
+def random_hostname():
+    return ''.join(hex(random.randint(0, 15))[2:] for _ in range(12))
+
+
 def create_bwrap_argv(config):
     my_home = os.path.realpath(os.path.expanduser('~'))
     mount_tasks = [
@@ -319,6 +326,7 @@ def create_bwrap_argv(config):
         MountTask(MountMode.TMPFS, my_home),
     ]
     env_tasks = {var: None for var in ['HOME', 'TERM', 'USER', 'WINEDEBUG']}
+    env_tasks['container'] = 'sandwine'
     unshare_args = ['--unshare-all']
 
     argv = ArgvBuilder()
@@ -326,11 +334,17 @@ def create_bwrap_argv(config):
     argv.add('bwrap')
     argv.add('--new-session')
 
+    # Hostname
+    hostname = random_hostname()
+    env_tasks['HOSTNAME'] = hostname
+    argv.add('--hostname', hostname)
+
     # Networking
     if config.network:
         unshare_args += ['--share-net']
         mount_tasks += [
-            MountTask(MountMode.BIND_RO, '/run/NetworkManager/resolv.conf', required=False)
+            MountTask(MountMode.BIND_RO, '/run/NetworkManager/resolv.conf', required=False),
+            MountTask(MountMode.BIND_RO, '/run/systemd/resolve/stub-resolv.conf', required=False),
         ]
 
     # Sound
@@ -408,16 +422,16 @@ def create_bwrap_argv(config):
                 mount_task.source = mount_task.target
 
             # NOTE: The X11 Unix socket will only show up later
-            keep_missing_target = X11Mode(
+            keep_missing_source = X11Mode(
                 config.x11) != X11Mode.NONE and mount_task.target == x11_unix_socket
 
-            if not os.path.exists(mount_task.target) and not keep_missing_target:
+            if not os.path.exists(mount_task.source) and not keep_missing_source:
                 if mount_task.required:
                     _logger.error(
-                        f'Path {mount_task.target!r} does not exist on the host, aborting.')
+                        f'Path {mount_task.source!r} does not exist on the host, aborting.')
                     sys.exit(1)
                 else:
-                    _logger.debug(f'Path {mount_task.target!r} does not exist on the host'
+                    _logger.debug(f'Path {mount_task.source!r} does not exist on the host'
                                   ', dropped from mount tasks.')
                     continue
 
@@ -434,6 +448,7 @@ def create_bwrap_argv(config):
 
     # Filter ${PATH}
     candidate_paths = os.environ['PATH'].split(os.pathsep)
+    candidate_paths.append('/usr/lib/wine')  # for wineserver on e.g. Debian
     available_paths = []
     for candidate_path in candidate_paths:
         candidate_path = os.path.realpath(candidate_path)
